@@ -283,7 +283,7 @@ Return a list of traces, possibly marked with <compliance-error>."
 
         (define (run-provides-modeling ipc port-instance)
           (%debug "run-provides-modeling... ~a\n" event)
-          (let* ((ipc (clone pc #:instance port-instance))
+          (let* ((ipc (clone pc #:instance port-instance #:statement #f))
                  (interface ((compose .type .ast) port-instance))
                  (modeling-names (modeling-names interface)))
             (parameterize ((%sut port-instance)
@@ -321,7 +321,13 @@ Return a list of traces, possibly marked with <compliance-error>."
               (and (not (equal? (cdr a) ((compose last (cut string-split <> #\.) cdr) b))) (cons a b)))
 
             (let* ((port-trail (port-trace->trail port-trace))
-                   (truncate? (and provides-trigger? blocked?))
+                   (port-length (length port-trail))
+                   (sut-length (length sut-trail))
+                   (port-next (and (> port-length sut-length)
+                                   (and=> (list-ref port-trail sut-length) cdr)))
+                   (truncate? (and provides-trigger?
+                                   blocked?
+                                   (not (equal? port-next "<illegal>"))))
                    (port-trail (if (not truncate?) port-trail
                                    ;; Check prefix only as long as trace is blocked
                                    (list-head port-trail (min (length port-trail)
@@ -398,18 +404,22 @@ Return a list of traces, possibly marked with <compliance-error>."
               (%debug "  exit 5\n")
               (let* ((port-acceptances (map first-non-match non-compliances))
                      (port-acceptances (delete-duplicates port-acceptances port-acceptance-equal?))
-                     (component-acceptance (and (pair? trace)
-                                                (or (and (pair? port-acceptances)
-                                                         (cadar port-acceptances))
-                                                    (and=> (.status pc) .ast)
-                                                    (trigger->component-trigger trigger))))
-                     (other-port-instances (runtime:other-port port-instance))
-                     (instance (.container other-port-instances))
+                     (component-acceptance
+                      (and (pair? trace)
+                           (or (and (pair? port-acceptances)
+                                    (cadar port-acceptances))
+                               (and=> (.status pc) .ast)
+                               (if (is-a? (%sut) <runtime:system>)
+                                   (trigger->system-trigger port-instance trigger)
+                                   (trigger->component-trigger trigger)))))
+                     (other-port-instance (runtime:other-port port-instance))
+                     (instance (.container other-port-instance))
                      (component-acceptance
                       (if (and component-acceptance (is-a? (%sut) <runtime:system>))
                           (trigger->system-trigger instance component-acceptance)
                           component-acceptance))
-                     (port-acceptances (make <acceptances> #:elements (map caar port-acceptances)))
+                     (port-acceptances (make <acceptances>
+                                         #:elements (map caar port-acceptances)))
                      (compliance-trigger
                       (and trigger
                            (null? sut-trail)
@@ -523,7 +533,7 @@ port return."
 
   (define (event-traces-alist pc)
     (define (event->label-traces pc event)
-      (let* ((pc (clone pc #:instance #f #:trail '()))
+      (let* ((pc (clone pc #:instance #f #:statement #f #:trail '()))
              (pc (reset-replies pc))
              (traces (parameterize ((%exploring? #t)
                                     (%liveness? 'component))
@@ -543,7 +553,7 @@ port return."
       (merge-alist2 alist async-alist)))
 
   (define (provides-event->label-traces pc event)
-    (let* ((pc (clone pc #:instance #f #:trail '()))
+    (let* ((pc (clone pc #:instance #f #:statement #f #:trail '()))
            (pc (reset-replies pc))
            (traces (parameterize ((%exploring? #t)
                                   (%liveness? 'component))
@@ -551,7 +561,7 @@ port return."
       (cons event traces)))
 
   (define (requires-event->label-traces pc event)
-    (let* ((pc (clone pc #:instance #f #:trail '()))
+    (let* ((pc (clone pc #:instance #f #:statement #f #:trail '()))
            (pc (reset-replies pc))
            (traces (parameterize ((%exploring? #t)
                                   (%liveness? 'component))
@@ -663,7 +673,7 @@ TRACES."
     (let* ((error (.status pc))
            (ast (or (and error (.ast error))
                     (let ((model (runtime:%sut-model)))
-                      (if (is-a? model <system>) model
+                      (if (or (is-a? model <system>) (is-a? model <foreign>)) model
                           (.behavior model))))))
       (if (and error
                (not (is-a? error <implicit-illegal-error>))
