@@ -38,7 +38,7 @@
 
   #:export (ast:wfc
             type-name
-            wfc:report-error))
+            wfc:report-message))
 
 ;;; Commentary:
 ;;;
@@ -52,19 +52,23 @@
 ;;; Code:
 
 (define (ast:wfc o)
-  (let ((errors (wfc o)))
+  (let* ((messages (wfc o))
+         (errors (filter (is? <error>) messages))
+         (messages (filter (is? <message>) messages)))
     (when (pair? errors)
-      (apply throw 'well-formedness-error errors)))
+      (apply throw 'well-formedness-error messages))
+    (when (pair? messages)
+       (for-each wfc:report-message messages)))
   o)
 
-(define (wfc:report-error o)
+(define-method (wfc:report-message (o <message>))
   (let* ((ast (.ast o))
          (loc (.location ast)))
-    (if loc
+    (if (not loc) (format (current-error-port) "error: ~a\n" (.message o))
         (format (current-error-port)
                 "~a:~a:~a: ~a: ~a\n"
-                (.file-name loc) (.line loc) (.column loc) (ast-name o) (.message o))
-        (format (current-error-port) "error: ~a\n" (.message o)))))
+                (.file-name loc) (.line loc) (.column loc)
+                (ast-name o) (.message o)))))
 
 (define (wfc-error o message)
   (make <error> #:ast o #:message message))
@@ -72,10 +76,13 @@
 (define (wfc-info o message)
   (make <info> #:ast o #:message message))
 
+(define (wfc-warning o message)
+  (make <warning> #:ast o #:message message))
+
 (define-method (wfc (o <model>))
   '())
 
-(define-method (wfc (o <interface>)) ;; is-a <model>
+(define-method (wfc (o <interface>))
   (append
    (re-definition o)
    (append-map wfc (ast:type* o))
@@ -88,6 +95,7 @@
 (define-method (wfc (o <foreign>))
   (append
    (re-definition o)
+   (append-map wfc (ast:port* o))
    (let* ((root (parent o <root>))
           (basename (ast:base-name root)))
      (if (not (equal? (string-join (ast:full-name o) "_") basename)) '()
@@ -141,7 +149,7 @@
 (define-method (wfc (o <type>))
   (re-definition o))
 
-(define-method (wfc (o <enum>)) ;; is-a <type>
+(define-method (wfc (o <enum>))
   (append
    (re-definition o)
    (let loop ((fields (ast:field* o)) (result '()))
@@ -156,7 +164,7 @@
        '())
    '()))
 
-(define-method (wfc (o <int>)) ;; is-a <type>
+(define-method (wfc (o <int>))
   (append (re-definition o)
           (let ((range (.range o)))
             (if (<= (.from range) (.to range)) '()
@@ -180,8 +188,9 @@
                                                 (string-join (map .name out-events) ", ")))
                           ,@(map (cut wfc-info <> (format #f "port defined here")) out-events))))))
            (else '())))
-   ;; TODO; do include async port in behavior
-   ))
+   (if (not (ast:async? o)) '()
+       (list
+        (wfc-warning o "dzn.async is deprecated, please update to defer.")))))
 
 
 (define-method (wfc (o <event>))
@@ -365,21 +374,21 @@
 (define-method (wfc (o <statement>))
   '())
 
-(define-method (wfc (o <compound>)) ;; is-a <statement>
+(define-method (wfc (o <compound>))
   (append
    (imperative-context o)
    (mixing-declarative-imperative o)
    (append-map wfc (ast:statement* o))))
 
-(define-method (wfc (o <declarative>)) ;; is-a <statement>
+(define-method (wfc (o <declarative>))
   '())
 
-(define-method (wfc (o <declarative-compound>)) ;; is-a <declarative>
+(define-method (wfc (o <declarative-compound>))
   (append
    (mixing-declarative-imperative o)
    (append-map wfc (ast:statement* o))))
 
-(define-method (wfc (o <guard>)) ;; is-a <declarative>
+(define-method (wfc (o <guard>))
   (define (otherwise-guard? o)
     (and (is-a? o <guard>)
          (is-a? (.expression o) <otherwise>)))
@@ -408,11 +417,11 @@
             (if (is-a? (.statement o) <statement>) (wfc (.statement o))
                 `(,(wfc-error o "statement expected"))))))
 
-(define-method (wfc (o <declarative-illegal>)) ;; is-a <declarative>
+(define-method (wfc (o <declarative-illegal>))
   ;; TODO; in source??
   '())
 
-(define-method (wfc (o <blocking>)) ;; is-a <declarative>
+(define-method (wfc (o <blocking>))
   (define (blocking o)
     (let ((model (parent o <model>)))
       (cond ((is-a? model <interface>)
@@ -423,7 +432,36 @@
             (else '()))))
   (append (blocking o) (wfc (.statement o))))
 
-(define-method (wfc (o <on>)) ;; is-a <declarative>
+(define (wfc-defer-argument var)
+  (if (is-a? var <undefined>)
+      `(,(wfc-error var (format #f "undefined identifier `~a'" (.name var))))
+      (let* ((variable (.variable var))
+             (name (.variable.name var))
+             (type (and=> variable .type)))
+        (cond
+         ((not (ast:member? variable))
+          `(,(wfc-error var (format
+                             #f
+                             "cannot use local variable `~a' as defer argument"
+                             name))
+            ,(wfc-info var (format #f "variable `~a' defined here" name))))
+         ((is-a? type <extern>)
+          `(,(wfc-error var (format
+                             #f
+                             "cannot use data variable `~a' as defer argument"
+                             name))
+            ,(wfc-info var (format #f "variable `~a' defined here" name))))
+         (else
+          '())))))
+
+(define-method (wfc (o <defer>))
+  (let ((arguments (ast:argument* o)))
+    (append
+     (if (not arguments) '()
+         (append-map wfc-defer-argument arguments))
+     (wfc (.statement o)))))
+
+(define-method (wfc (o <on>))
   (define (on o)
     (append
      (let ((parent (parent (.parent o) <on>)))
@@ -436,38 +474,38 @@
    (if (is-a? (.statement o) <statement>) (wfc (.statement o))
        `(,(wfc-error o "statement expected")))))
 
-(define-method (wfc (o <imperative>)) ;; is-a <statement>
+(define-method (wfc (o <imperative>))
   '())
 
-(define-method (wfc (o <variable>)) ;; is-a <imperative>
+(define-method (wfc (o <variable>))
   (append
    (imperative-context o)
    (re-definition o)
    (assign o)))
 
-(define-method (wfc (o <action>)) ;; is-a <imperative>
+(define-method (wfc (o <action>))
   (append
    (action o)
    (call-context o)))
 
-(define-method (wfc (o <action-or-call>)) ;; is-a <imperative>
+(define-method (wfc (o <action-or-call>))
   (append
    (call-context o)
    '()))
 
-(define-method (wfc (o <assign>)) ;; is-a <imperative>
+(define-method (wfc (o <assign>))
   (append
    (assign o)
    (imperative-context o)))
 
-(define-method (wfc (o <call>)) ;; is-a <imperative>
+(define-method (wfc (o <call>))
   (append
    (defined-function o)
    (call-context o)
    (tail-recursion o)
    (wfc (.arguments o))))
 
-(define-method (wfc (o <if>)) ;; is-a <imperative>
+(define-method (wfc (o <if>))
   (let* ((expression (.expression o))
          (wfce (wfc expression)))
     (append wfce
@@ -477,7 +515,7 @@
             (wfc (.then o))
             (if (.else o) (wfc (.else o)) '()))))
 
-(define-method (wfc (o <illegal>)) ;; is-a <imperative>
+(define-method (wfc (o <illegal>))
   (define (illegal o)
     (let ((model (parent o <model>)))
       (cond ((and (is-a? model <interface>) (parent (.parent o) <function>))
@@ -499,7 +537,7 @@
    (imperative-context o)
    (illegal o)))
 
-(define-method (wfc (o <reply>)) ;; is-a <imperative>
+(define-method (wfc (o <reply>))
   (append
    (imperative-context o)
    (if (.expression o) (wfc (.expression o)) '())
@@ -561,7 +599,7 @@
             `(,(wfc-error o (format #f "type mismatch: no event with reply type `~a'"
                                     reply-type-name)))))))))
 
-(define-method (wfc (o <return>)) ;; is-a <imperative>
+(define-method (wfc (o <return>))
   (let* ((wfce (if (.expression o) (wfc (.expression o)) '()))
          (function (parent o <function>))
          (function-type (and function (ast:type function)))
@@ -576,11 +614,11 @@
                                            (type-name return-type)))))
                   (else '())))))
 
-(define-method (wfc (o <the-end>))  ;; is-a <statement> ;; not in source
+(define-method (wfc (o <the-end>))
   '())
-(define-method (wfc (o <the-end-blocking>))  ;; is-a <statement> ;; not in source
+(define-method (wfc (o <the-end-blocking>))
   '())
-(define-method (wfc (o <voidreply>))  ;; is-a <statement> ;; not in source
+(define-method (wfc (o <voidreply>))
   '())
 
 (define-method (wfc (o <trigger>))
@@ -671,24 +709,24 @@
   (typed-expression (.expression o) <bool>))
 
 (define-method (typed-binary (o <binary>) (type <class>))
-  (let* ((expr1 (.left o))
-         (expr1-wfc (typed-expression expr1 type))
-         (expr2 (.right o))
-         (expr2-wfc (typed-expression expr2 type)))
-    (append expr1-wfc expr2-wfc)))
+  (let* ((left (.left o))
+         (left-wfc (typed-expression left type))
+         (right (.right o))
+         (right-wfc (typed-expression right type)))
+    (append left-wfc right-wfc)))
 
 (define-method (binary-equal-no-extern-type (o <binary>))
-  (let* ((expr1 (.left o))
-         (expr1-wfc (no-extern-expression expr1))
-         (expr1-type (ast:type expr1))
-         (expr2 (.right o))
-         (expr2-wfc (no-extern-expression expr2))
-         (expr2-type (ast:type expr2)))
-    (cond ((or (pair? expr1-wfc) (pair? expr2-wfc)) (append expr1-wfc expr2-wfc))
-          ((not (equal-type? expr1-type expr2-type))
+  (let* ((left (.left o))
+         (left-wfc (no-extern-expression left))
+         (left-type (ast:type left))
+         (right (.right o))
+         (right-wfc (no-extern-expression right))
+         (right-type (ast:type right)))
+    (cond ((or (pair? left-wfc) (pair? right-wfc)) (append left-wfc right-wfc))
+          ((not (equal-type? left-type right-type))
            `(,(wfc-error o (format #f "type mismatch in binary operator: `~a' versus `~a'"
-                                   (type-name expr1-type)
-                                   (type-name expr2-type)))))
+                                   (type-name left-type)
+                                   (type-name right-type)))))
           (else '()))))
 
 (define-method (wfc (o <and>)) (typed-binary o <bool>))
@@ -760,7 +798,7 @@
 
 (define-method (tail-recursion (o <call>))
   (let ((function (parent o <function>)))
-    (if (or (not function) (not (.recursive function))) '()
+    (if (or (not function) (not (.recursive? function))) '()
         (let* ((continuation ((compose car wfc:continuation) o))
                (continuation (if (or (parent o <assign>)
                                      (parent o <reply>)
@@ -793,9 +831,9 @@
 
 (define-method (re-definition (o <declaration>))
   (let* ((name (ast:name o))
-         (scope (decl-scope o))
+         (scope (parent (.parent o) <scope>))
          (previous (and scope (ast:lookup scope name)))
-         (previous-scope (and previous (decl-scope previous))))
+         (previous-scope (and previous (parent (.parent previous) <scope>))))
     (if (and scope
              previous
              (ast:eq? scope previous-scope)
@@ -805,28 +843,6 @@
           ,(wfc-info previous (format #f "previous `~a' definition here" (ast:name previous))))
         '())))
 
-
-(define-method (decl-scope (o <declaration>))
-  (and=> (.parent o) (cut parent <> <scope>)))
-
-(define-method (decl-scope (o <event>))
-  (parent o <interface>))
-
-(define-method (decl-scope (o <instance>))
-  (parent o <system>))
-
-(define-method (decl-scope (o <port>))
-  (parent o <component-model>))
-
-(define-method (decl-scope (o <formal>))
-  (.parent o))
-
-(define-method (decl-scope (o <function>))
-  (parent o <behavior>))
-
-(define-method (decl-scope (o <variable>))
-  (or (parent o <compound>) (parent o <behavior>)))
-
 (define-method (assign (o <ast>))
   (let* ((assign-type (ast:type o))
          (expression (.expression o))
@@ -1167,52 +1183,6 @@
                (is-a? p <guard>)
                (is-a? p <on>)))
       `(,(wfc-error o (format #f "~a value discarded" class))))
-     ((and (not (is-a? (ast:type o) <void>))
-           (not
-            (or (is-a? p <assign>)
-                (is-a? p <variable>)
-                (is-a? p <reply>)
-                (is-a? p <return>)
-                (is-a? p <if>)))
-           (not
-            (and (or (is-a? p <equal>)
-                     (is-a? p <not-equal>))
-                 (or (is-a? grand-parent <if>)
-                     (is-a? grand-parent <reply>)
-                     (is-a? grand-parent <return>))
-                 (ast:eq? (.left p) o)))
-           (not
-            (and (or (is-a? p <and>)
-                     (is-a? p <or>))
-                 (or (is-a? grand-parent <if>)
-                     (is-a? grand-parent <reply>)
-                     (is-a? grand-parent <return>))
-                 (ast:eq? (.left p) o)))
-           (not
-            (and (is-a? p <not>)
-                 (or (is-a? grand-parent <if>)
-                     (is-a? grand-parent <reply>)
-                     (is-a? grand-parent <return>))
-                 (ast:eq? (.expression p) o)))
-           (not
-            (and (or (is-a? p <equal>)
-                     (is-a? p <not-equal>))
-                 (or (is-a? grand-parent <and>)
-                     (is-a? grand-parent <or>))
-                 (or (is-a? great-grand-parent <if>)
-                     (is-a? great-grand-parent <reply>)
-                     (is-a? great-grand-parent <return>))
-                 (ast:eq? (.left grand-parent) p)))
-           (not
-            (and (is-a? p <not>)
-                 (or (is-a? grand-parent <and>)
-                     (is-a? grand-parent <or>))
-                 (let ((great-grand-parent (.parent grand-parent)))
-                   (or (is-a? great-grand-parent <if>)
-                       (is-a? great-grand-parent <reply>)
-                       (is-a? great-grand-parent <return>)))
-                 (ast:eq? (.left grand-parent) p))))
-      `(,(wfc-error o (format #f "~a used in a complex expression" class))))
      (else '()))))
 
 (define-method (imperative-context (o <ast>))
@@ -1243,18 +1213,18 @@
       `(,(wfc-error o (format #f "system composition of `~a' is recursive" (type-name (.name o)))))
       '()))
 
-(define-method (required-instances (o <instance>) (s <system>))
+(define-method (requires-instances (o <instance>) (s <system>))
   (let* ((instances (ast:instance* s))
          (bindings (ast:binding* s))
          (component (.type o))
-         (required-ports (and component (ast:requires-port* component)))
+         (requires-ports (and component (ast:requires-port* component)))
          (left-bindings (filter (lambda (b) (equal? (.name o) (.instance.name (.left b))))
                                 bindings))
          (right-bindings (filter (lambda (b) (equal? (.name o) (.instance.name (.right b))))
                                  bindings))
-         (left-required-bindings (filter (lambda (b) (find (lambda (p) (equal? (.name p) (.port.name (.left b)))) required-ports))
+         (left-required-bindings (filter (lambda (b) (find (lambda (p) (equal? (.name p) (.port.name (.left b)))) requires-ports))
                                          left-bindings))
-         (right-required-bindings (filter (lambda (b) (find (lambda (p) (equal? (.name p) (.port.name (.right b)))) required-ports))
+         (right-required-bindings (filter (lambda (b) (find (lambda (p) (equal? (.name p) (.port.name (.right b)))) requires-ports))
                                           right-bindings))
          (left-instances (filter (lambda (i) (find (lambda (b) (equal? (.name i) (.instance.name (.right b)))) left-required-bindings))
                                  instances))
@@ -1272,7 +1242,7 @@
 
 (define-method (cyclic-bindings (o <system>))
   (let* ((instances (ast:instance* o))
-         (required-alist (map (cut required-instances <> o) instances)))
+         (required-alist (map (cut requires-instances <> o) instances)))
     (append-map (lambda (i) (if (find (cut ast:eq? i <>) (all-required i o required-alist))
                                 `(,(wfc-error i (format #f "instance `~a' is in a cyclic binding" (.name i))))
                                 '()))
