@@ -47,14 +47,14 @@
 (define-public dezyne
   (package
     (name "dezyne")
-    (version #!dezyne!# "2.16.0.rc3")
+    (version #!dezyne!# "2.16.1.rc1")
     (source
      (origin
        (method url-fetch)
        (uri (string-append "https://dezyne.org/download/dezyne/"
                            name "-" version ".tar.gz"))
        (sha256
-        (base32 #!dezyne!# "03dwr6i33jk8yinvbz14gsd6phv5q9v6qrkb3xkrgh19y7n0ffnm"))))
+        (base32 #!dezyne!# "08qw9d1rv021hbxzjb3kz51xzk99l7lh53h0pk9k0wlv4cbzb3bp"))))
     (inputs (list bash-minimal
                   guile-3.0-latest
                   guile-json-4
