@@ -1,6 +1,7 @@
+#! /usr/bin/env node
 // Dezyne --- Dezyne command line tools
 //
-// Copyright © 2016, 2021 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+// Copyright © 2016, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 //
 // This file is part of Dezyne.
 //
@@ -21,14 +22,25 @@
 //
 // Code:
 
-var dzn = typeof (dzn) !== undefined && dzn ? dzn : require (__dirname + '/dzn/runtime');
-dzn.extend (dzn, require (__dirname + '/dzn/hello_injected'));
+process.env.NODE_PATH += ':' + __dirname;
+process.env.NODE_PATH += ':' + __dirname + '/../../javascript';
+require("module").Module._initPaths();
+
+function node_p () {return typeof (module) !== 'undefined';}
+function have_dzn_p () {return typeof (dzn) !== 'undefined' && dzn;}
+
+dzn = have_dzn_p () ? dzn : require ('dzn/runtime');
+dzn.extend (dzn, require ('hello_injected'));
 
 function main() {
   var loc = new dzn.locator();
   var rt = new dzn.runtime();
-  var sut = new dzn.hello_injected(loc.set(rt), 'sut');
-  sut.t.out.f = function() {console.error('sut.m.t.f -> <external>.t.f\n');};
+  var sut = new dzn.hello_injected(loc.set(rt), {name:'sut'});
+  sut.t._dzn.meta.requires = {name: 't', component: null};
+  sut.t._dzn.meta.provides.name = '';
+  sut.t._dzn.meta.requires.name = 't';
+
+  sut.t.out.f = function() {console.error('sut.m.t.f -> <external>.t.f');};
 
   sut.t.in.e();
 }
